char    RedShift;
long    RedTable [256];
char    GreenShift;
long    GreenTable [256];
char    BlueShift;
long    BlueTable [256];

//Get the position of the first bit set
char FirstBit (long Number)
{
  char Result = 0;

  if (!Number) return 0;

  //Find first bit
  while (!(Number & 1)) {
    Result++;
    Number >>= 1;
  }

  return Result;
}

//Initialises the RGB tables
void InitRGBTables()
{
  //Get masks and shift them to the right
  long RedMask = Mode->pf.dwRBitMask;
  long GreenMask = Mode->pf.dwGBitMask;
  long BlueMask = Mode->pf.dwBBitMask;

  RedShift = FirstBit (RedMask);
  GreenShift = FirstBit (GreenMask);
  BlueShift = FirstBit (BlueMask);

  RedMask >>= RedShift;
  GreenMask >>= GreenShift;
  BlueMask >>= BlueShift;

  //Calculate pixel values for all colour components
  for (long Index = 0; Index < 256; Index++) {
    RedTable [Index] = (Index * RedMask / 255) << RedShift;
    GreenTable [Index] = (Index * GreenMask / 255) << GreenShift;
    BlueTable [Index] = (Index * BlueMask / 255) << BlueShift;
  }
}

//Converts an RGB triplet into an RGB pixel
long Convert (unsigned char *Ofs)
{
  unsigned char b = *Ofs;
  unsigned char g = *(Ofs + 1);
  unsigned char r = *(Ofs + 2);

  return (RedTable [r] | GreenTable [g] | BlueTable [b]);
}

//Loads bitmaps from files
int LoadBitmaps ()
{
  FILE *BMPFile;

  //Load height bitmap
  BMPFile = fopen ("HEIGHT.BMP", "rb");
  fread (&Height_BitmapFileHeader, sizeof (BITMAPFILEHEADER), 1, BMPFile);
  fread (&Height_BitmapInfoHeader, sizeof (BITMAPINFOHEADER), 1, BMPFile);
  if ((Height_BitmapInfoHeader.biWidth | Height_BitmapInfoHeader.biHeight) != 256) return 0;
  Height_Bitmap = (unsigned char *)malloc (65536);
  fseek (BMPFile, Height_BitmapFileHeader.bfOffBits, SEEK_SET);
  fread (Height_Bitmap, 256, 256, BMPFile);
  fclose (BMPFile);

  //Load colour bitmap
  BMPFile = fopen ("COLOUR.BMP", "rb");
  fread (&Colour_BitmapFileHeader, sizeof (BITMAPFILEHEADER), 1, BMPFile);
  fread (&Colour_BitmapInfoHeader, sizeof (BITMAPINFOHEADER), 1, BMPFile);
  if ((Colour_BitmapInfoHeader.biWidth | Colour_BitmapInfoHeader.biHeight) != 256) return 0;
  unsigned char *Bitmap = (unsigned char *)malloc (65536 * 3);
  fseek (BMPFile, Colour_BitmapFileHeader.bfOffBits, SEEK_SET);
  fread (Bitmap, 256 * 3, 256, BMPFile);
  fclose (BMPFile);

  //Convert to RGB
  InitRGBTables ();
  Colour_Bitmap = (long *)malloc (65536 << 2);
  unsigned char *Ofs1 = Bitmap;
  long *Ofs2 = Colour_Bitmap;
  for (long Pos = 0; Pos < 65536; Pos++) {
    *Ofs2 = Convert (Ofs1);
    Ofs1 += 3;
    Ofs2++;
  }

  free (Bitmap);

  return -1;
}