//Callback function to enumerate display modes
HRESULT WINAPI EnumModesCallback (LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext)
{
  //Add RGB modes to list
  int bpp = lpDDSurfaceDesc->ddpfPixelFormat.dwRGBBitCount;
  if (bpp == 16 || bpp == 24 || bpp == 32) {
    Modes = (VIDEOMODE *)realloc (Modes, sizeof (VIDEOMODE) * (NumModes + 1));
    Modes [NumModes].x = lpDDSurfaceDesc->dwWidth;
    Modes [NumModes].y = lpDDSurfaceDesc->dwHeight;
    Modes [NumModes].pf = lpDDSurfaceDesc->ddpfPixelFormat;
    NumModes++;
  }

  //Continue enumeration
  return DDENUMRET_OK;
}

//Initialises primary flippable surface with backbuffer attached
int CreatePrimarySurface (VIDEOMODE *VideoMode)
{
  DDSCAPS       ddsCaps;
  HRESULT       Result;

  //Set cooperative level to run full screen, allowing video mode changes
  Result = DDObject->SetCooperativeLevel (WindowHandle, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
  if (Result != DD_OK) return 0;

  //Set video mode
  Result = DDObject->SetDisplayMode (VideoMode->x, VideoMode->y, VideoMode->pf.dwRGBBitCount);
  if (Result != DD_OK) return 0;

  //Set up Surface Description
  memset (&SurfaceDesc, 0, sizeof (SurfaceDesc));
  SurfaceDesc.dwSize = sizeof (SurfaceDesc);
  SurfaceDesc.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
  SurfaceDesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
  SurfaceDesc.dwBackBufferCount = 1;

  //Create primary surface
  Result = DDObject->CreateSurface (&SurfaceDesc, &DDPrimary, NULL);
  if (Result != DD_OK) return 0;

  //Get the pointer to the back buffer
  ddsCaps.dwCaps = DDSCAPS_BACKBUFFER;
  Result = DDPrimary->GetAttachedSurface (&ddsCaps, &DDBack);
  if (Result != DD_OK) return 0;

  //Get surface condition
  Result = DDPrimary->GetSurfaceDesc (&SurfaceDesc);
  if (Result != DD_OK) return 0;

  //Calculate offsets
  Lines = (long *)malloc (VideoMode->y * sizeof (long));
  long Ofs = 0;
  for (long Line = 0; Line < VideoMode->y; Line++) {
    Lines [Line] = Ofs;
    Ofs += SurfaceDesc.lPitch;
  }
  Bpp = VideoMode->pf.dwRGBBitCount >> 3;
  Columns = (long *)malloc (VideoMode->x * sizeof (long));
  Ofs = 0;
  for (long Column = 0; Column < VideoMode->x; Column++) {
    Columns [Column] = Ofs;
    Ofs += Bpp;
  }

  //Successfull
  return -1;
}

//Initializes DirectDraw
int InitDirectDraw ()
{
  HRESULT   Result;

  //Initialize main DirectDraw object
  Result = DirectDrawCreate (NULL, &DDObject, NULL);
  if (Result != DD_OK) return 0;

  //Enumerate display modes
  DDObject->EnumDisplayModes (NULL, NULL, NULL, EnumModesCallback);

  //Successfull
  return -1;
}

//Releases DirectDraw
void CloseDirectDraw ()
{
  //All of DirectDraw
  if (DDObject) {
    //Primary surface
    if (DDPrimary) {
      DDPrimary->Release ();
      DDPrimary = NULL;
      DDBack = NULL;
    }
    //DirectDraw itself
    DDObject->Release ();
    DDObject = NULL;
  }
}