/*********************************************************************
** Demonstration of programming the a voxel landscape through       **
** DirectDraw in any available resolution                           **
**   by Steven Don                                                  **
**                                                                  **
** For questions, feel free to e-mail me.                           **
**                                                                  **
**    shd@earthling.net                                             **
**    http://shd.home.ml.org                                        **
**                                                                  **
**********************************************************************
** NOTE: This example only works with Microsoft Visual C++ for Win  **
**       Be sure to link to DDRAW.LIB when compiling.               **
**       Also, the file DDRAW.H should be in your include path      **
**                                                                  **
** You'll need an 8bit grayscale file for the height field and a    **
** 24bit colour image for the colour data. Both must be 256x256 in  **
** size.                                                            **
**********************************************************************
** To quit this program, just press Alt-F4 like with any other Win  **
** application.                                                     **
*********************************************************************/
#include "stdio.h"
#include "windows.h"
#include "ddraw.h"
#include "math.h"

//Handles for Windows stuff
HINSTANCE           InstanceHandle;
HWND                WindowHandle;
HWND                BoxHandle;
HWND                ButtonHandle;
MSG                 Message;

//Structures
typedef struct {
  long              x, y;
  DDPIXELFORMAT     pf;
} VIDEOMODE;

//For selecting video mode
VIDEOMODE           *Modes;
VIDEOMODE           *Mode;
long                NumModes = 0;
long                Bpp = 0;
long                *Lines;
long                *Columns;
long                DComp1 = 0;
long                DComp2 = 0;
long                DComp3 = 0;

//Handles for DirectX stuff
LPDIRECTDRAW        DDObject;
DDSURFACEDESC       SurfaceDesc;
LPDIRECTDRAWSURFACE DDPrimary, DDBack;
char                *VideoMemory;

//Landscape information
BITMAPFILEHEADER    Height_BitmapFileHeader;
BITMAPINFOHEADER    Height_BitmapInfoHeader;
unsigned char       *Height_Bitmap;
BITMAPFILEHEADER    Colour_BitmapFileHeader;
BITMAPINFOHEADER    Colour_BitmapInfoHeader;
long                *Colour_Bitmap;

//Player info
unsigned short      x = 0x8000;
unsigned short      y = 0x8000;
float               Angle = 1.84f;
long                Height;

//Global indicator
int                 Status = 0;

#include "Voxels - WindowStuff.cpp"
#include "Voxels - DirectXStuff.cpp"
#include "Voxels - BitmapStuff.cpp"
#include "Voxels - Frame.cpp"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
  //Store instance
  InstanceHandle = hInstance;

  //Initialize window
  InitWindow ();

  //Initialize DirectDraw
  if (!InitDirectDraw ()) {
    MessageBox (NULL, "Could not initialize DirectDraw!", NULL, MB_OK | MB_ICONERROR);
    CloseDirectDraw ();
    return 0;
  }

  //Message loop
  do {
    switch (Status) {
      //Initialise menu window
      case 0 :
        InitMenuWindow ();
        Status++;
        break;
      //Initialise game
      case 2 :
        //Retrieve screen mode
        Mode = &Modes [SendMessage (BoxHandle, CB_GETCURSEL, NULL, NULL)];
        //Switch to game window
        DestroyWindow (WindowHandle);
        InitGameWindow ();
        //Active DirectDraw
        if (!CreatePrimarySurface (Mode)) {
          DestroyWindow (WindowHandle);
          MessageBox (NULL, "Could not initialize DirectDraw!", NULL, MB_OK | MB_ICONERROR);
          Status = 4;
          break;
        }
        //Load bitmaps
        if (!LoadBitmaps ()) {
          DestroyWindow (WindowHandle);
          MessageBox (NULL, "Error loading bitmaps!", NULL, MB_OK | MB_ICONERROR);
          Status = 4;
          break;
        }
        //Set resolution compensation
        DComp1 = 5 * Mode->y;
        DComp2 = Mode->y >> 1;
        DComp3 = (long)(log (Mode->y / 4) / log (2));
        Status = 3;
        break;
      //Do game
      case 3 :
        //Adjust camera height according to height map
        Height = Height_Bitmap [(y & 0xFF00) + (x >> 8)];
        //Draw voxels
        DrawView ();
        //Movement
        if (GetAsyncKeyState (VK_UP)) {
          x += (unsigned short)(cos (Angle) * 256.0f);
          y += (unsigned short)(sin (Angle) * 256.0f);
        }
        if (GetAsyncKeyState (VK_LEFT)) {
          Angle -= 0.05f;
        }
        if (GetAsyncKeyState (VK_RIGHT)) {
          Angle += 0.05f;
        }
        if (GetAsyncKeyState (VK_DOWN)) {
          x -= (unsigned short)(cos (Angle) * 256.0f);
          y -= (unsigned short)(sin (Angle) * 256.0f);
        }
        break;
    }
    //Message loop
    while (PeekMessage(&Message, NULL, 0, 0, PM_REMOVE)) {
      TranslateMessage (&Message);
      DispatchMessage (&Message);
    }
  } while (Status != 4);

  //Close DirectX
  CloseDirectDraw ();

  //Remove window
  DestroyWindow (WindowHandle);

  return Message.wParam;
}